package com.jln.system.web.action.mqtt;

import com.jln.common.enums.MqttEnum;
import com.jln.system.web.action.mqtt.mqttService.SSL;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Map;

public class XBMqttDataUtil {

    private static Logger logger = LoggerFactory.getLogger(XBMqttDataUtil.class);

    private MqttClient mqttClient;
    private Map<String, String> configParams;

    public XBMqttDataUtil( Map<String, String> configParams) {
        this.configParams = configParams;
        try {
            getMqttClient(configParams);
            } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 构造函数
     *
     * @throws MqttException
     */
    public void getMqttClient(Map<String, String> configParams) {
        // 访问ip：port  tcp://   or  ssl://
        String host = configParams.get("MQTT_HOST");
        // 客户端id 保持唯一
        String clientId = configParams.get("MQTT_CLIENTID");
        // 用户名
        String userName = configParams.get("MQTT_USERNAME");
        // 密码
        String password = configParams.get("MQTT_PASSWORD");
        // MemoryPersistence设置clientid的保存形式，默认为以内存保存
        try {
            mqttClient = new MqttClient(host, clientId, new MemoryPersistence());
            MqttConnectOptions options = new MqttConnectOptions();
            //使用ssl方式连接
            InputStream crt = null;
            if (host.startsWith("ssl")) {
                String urlPath=System.getProperty("user.dir")+ "/cacert/cacert.cer";
                crt = new FileInputStream(urlPath);
                options.setSocketFactory(new SSL().getSSLSocktet(crt));
            }

            options.setCleanSession(false);
            options.setAutomaticReconnect(true);//设置自动重连
            options.setUserName(userName);
            options.setPassword(password.toCharArray());
            // 设置超时时间
            options.setConnectionTimeout(1000);
            //支持同时发送的消息数为1000
            options.setMaxInflight(1000);
            // 设置会话心跳时间
            options.setKeepAliveInterval(60);
            mqttClient.setCallback(new PushCallback());
            mqttClient.connect(options);
            logger.info("SSL连接MQTT成功");
        } catch (Exception e) {
            logger.error("SSL连接MQTT失败" + e.getMessage());
        }
    }

    public void close() {
        try {
            mqttClient.close();
        } catch (MqttException e) {
            logger.error(e.getMessage());
        }
    }

    /**
     * 发布数据
     *
     * @param
     * @param data
     * @throws Exception
     */
    public synchronized void publish( String data) {
        try {
            if (!mqttClient.isConnected()){
                getMqttClient(configParams);
            }
            MqttMessage mqttMessage = new MqttMessage();
//            mqttMessage.setPayload(new Gson().toJson ( data ).getBytes ( "UTF-8" ));
            mqttMessage.setPayload(data.getBytes( "UTF-8"));
            // QoS：发布消息的服务质量，即：保证消息传递的次数（消费者收到的次数）
            // 0：最多一次，即：<=1；每个消息只发一次，也不会缓存下来。
            // 1：至少一次，即：>=1；一直发送确保消费者至少收到一次，发送失败会缓存下来。
            // 2：一次，即：=1       一直发送确保消费者只能收到一次；发送失败会缓存下来 。
            mqttMessage.setQos(1);
            //消费者断开连接后是否接受离线消息
            mqttMessage.setRetained(true);
            String topic = configParams.get(MqttEnum.MQTT_TOPIC.getMqttName());
            mqttClient.publish(topic, mqttMessage);
            logger.info("MQTT发布主题:" + topic + ":数据成功" + mqttMessage);
        } catch (Exception e) {
            logger.error(e.getMessage());
        } finally {
            try {
                mqttClient.close();
            } catch (MqttException e) {
                logger.error(e.getMessage());
            }
        }

    }

    /**
     * 断开连接
     */
    public void destroy() {
        try {
            this.mqttClient.disconnect();
            logger.error("mqtt 手动断开连接！！！");
        } catch (MqttException e) {
            logger.error("手动断开连接报错error={}", e.getMessage());
        }
    }
}
